package simple.opencard;

/**
 * Titre :        Simple
 * Description :  A simple JavaCard applet and its OCF associated CardService and CardServiceFactory class
 * Copyright :    Copyright (c) 2002
 * Socit :
 * @author Unknown
 * @version 1.0
 */

//Listing Two

import java.io.*;
import opencard.core.terminal.*;
import opencard.core.service.*;
import opencard.opt.service.*;

public class SimpleStringCardService extends CardService
{
	// APDU to select the root of the card file system
  private final byte selectRoot[] = {(byte)0xA0, (byte)0xA4, (byte)0x00, (byte)0x00, (byte)0x02, (byte)0x3F, (byte)0x00};
	// APDU to select the SimpleString applet's AID
  private final byte selectApp[] = {(byte)0xA0, (byte)0xA4, (byte)0x04, (byte)0x00, (byte)0x07, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x07};
	// response status word: everything OK
  protected final static int OK = 0x9000;
	// response status word: string length
  protected final static byte STRLEN_PREFIX = 0x67;
	// if TRUE, the applet is selected
  boolean appletSelected;
  public SimpleStringCardService(){
    super();
  }
  protected void initialize(CardServiceScheduler scheduler, SmartCard card, boolean blocking)
      throws CardServiceException{
    super.initialize(scheduler, card, blocking);
    appletSelected = false;
  }

  public void writeString(String inString)
      throws CardServiceUnexpectedResponseException{
    try{
      if (!appletSelected) selectApplet();
      // convert string to bytes
      byte[] inArray = inString.getBytes();
      // set up the write command APDU
      byte[] write = new byte[(5 + inArray.length)];
      // APDU header
      write[0] = (byte) 0x00;			// CLA
      write[1] = (byte) 0x10;			// INS
      write[2] = (byte) 0x00;			// P1
      write[3] = (byte) 0x00;			// P2
      write[4] = (byte) inArray.length;	// Le
      // APDU data
      System.arraycopy(inArray, 0, write, 5, inArray.length);
      allocateCardChannel();
      // send APDU
      ResponseAPDU response = sendAPDU(write);
    // check the response
    switch (response.sw() & 0xFFFF) {
      case OK:
	return;
      default:
	throw new CardServiceUnexpectedResponseException("Unexpected Status Words: 0x" + Integer.toHexString((short) (response.sw() & 0xFFFF)));
    }
  } catch(Exception e){
    e.printStackTrace();
  } finally {
    releaseCardChannel();
  }
} // end writeString method

public String readString()
    throws CardServiceException {
  try{
    if (!appletSelected) selectApplet();
    allocateCardChannel();

    // step 1: the read command APDU with expected response data length Le = 0
    byte read[] = {(byte)0x00, (byte)0x20, (byte)0x00, (byte)0x00, (byte)0x00};
    // send
    ResponseAPDU response = sendAPDU(read);

    // step 2: the card responds with sw2 equal to string length
    if ((response.sw1() & 0xFF) != STRLEN_PREFIX)
      throw new CardServiceUnexpectedResponseException("Unexpected Status Words: 0x" + Integer.toHexString((short) (response.sw() & 0xFFFF)));
    // response sw2 = length of the stored string

// step 3: the read command APDU, response data length Le = sw2
read[4] = response.sw2();
// send again
response = sendAPDU(read);
// check the response
switch (response.sw() & 0xFFFF)
{
case OK:
// if sw = 0x9000, return the data
return new String(response.data());
default:
throw new CardServiceUnexpectedResponseException("Unexpected Status Words: 0x" + Integer.toHexString((short) (response.sw() & 0xFFFF)));
}
} catch(Exception e)
{
e.printStackTrace();
throw new CardServiceException();
} finally
{
releaseCardChannel();
}
} // end readString method

public void selectApplet() throws CardServiceException
{
try
{
allocateCardChannel();
sendAPDU(selectRoot);
sendAPDU(selectApp);
} catch(Exception e)
{
e.printStackTrace();
throw new CardServiceException();
} finally
{
releaseCardChannel();
}
appletSelected = true;
return;
} // end selectApplet method
	// method for sending APDUs to the card
private ResponseAPDU sendAPDU(byte[] apdu) throws CardTerminalException
{
// Set up the command APDU
CommandAPDU commandAPDU = new CommandAPDU(apdu);
ResponseAPDU responseAPDU = getCardChannel().sendCommandAPDU(commandAPDU);
return (responseAPDU);
}
} // end SimpleStringCardService class
